package backend

import (
	"context"
	"errors"
	"io/ioutil"
	"log"
	"strings"

	ldaputil "git.autistici.org/ai3/go-common/ldap"
	"gopkg.in/ldap.v2"
)

type LDAPQueryConfig struct {
	// SearchBase, SearchFilter and Scope define parameters for
	// the LDAP search. The search should return a single object.
	// SearchBase and SearchFilter can contain the string "%s",
	// which will be replaced with the username before performing
	// a query.
	SearchBase   string `yaml:"search_base"`
	SearchFilter string `yaml:"search_filter"`
	Scope        string `yaml:"scope"`

	// Attr is the LDAP attribute holding the encrypted user key.
	Attr string `yaml:"attr"`
}

// Valid returns an error if the configuration is invalid.
func (c *LDAPQueryConfig) Valid() error {
	if c.SearchBase == "" {
		return errors.New("empty search_base")
	}
	if c.SearchFilter == "" {
		return errors.New("empty search_filter")
	}
	if c.Scope != "one" && c.Scope != "sub" {
		return errors.New("unknown scope")
	}
	if c.Attr == "" {
		return errors.New("empty attr")
	}
	return nil
}

func (c *LDAPQueryConfig) searchRequest(username string) *ldap.SearchRequest {
	u := ldap.EscapeFilter(username)
	base := strings.Replace(c.SearchBase, "%s", u, -1)
	filter := strings.Replace(c.SearchFilter, "%s", u, -1)
	scope := ldap.ScopeWholeSubtree
	if c.Scope == "one" {
		scope = ldap.ScopeSingleLevel
	}
	return ldap.NewSearchRequest(
		base,
		scope,
		ldap.NeverDerefAliases,
		0,
		0,
		false,
		filter,
		[]string{c.Attr},
		nil,
	)
}

// LDAPConfig holds the global configuration for the LDAP user backend.
type LDAPConfig struct {
	URI        string           `yaml:"uri"`
	BindDN     string           `yaml:"bind_dn"`
	BindPwFile string           `yaml:"bind_pw_file"`
	Query      *LDAPQueryConfig `yaml:"query"`
}

// Valid returns an error if the configuration is invalid.
func (c *LDAPConfig) Valid() error {
	if c.URI == "" {
		return errors.New("empty uri")
	}
	if c.BindDN == "" {
		return errors.New("empty bind_dn")
	}
	if c.BindPwFile == "" {
		return errors.New("empty bind_pw_file")
	}
	if c.Query == nil {
		return errors.New("missing query configuration")
	}
	return c.Query.Valid()
}

type ldapBackend struct {
	config *LDAPConfig
	pool   *ldaputil.ConnectionPool
}

func NewLDAPBackend(config *LDAPConfig) (*ldapBackend, error) {
	// Validate configuration.
	if err := config.Valid(); err != nil {
		return nil, err
	}

	// Read the bind password.
	bindPw, err := ioutil.ReadFile(config.BindPwFile)
	if err != nil {
		return nil, err
	}

	// Connect.
	pool, err := ldaputil.NewConnectionPool(config.URI, config.BindDN, strings.TrimSpace(string(bindPw)), 5)
	if err != nil {
		return nil, err
	}

	return &ldapBackend{
		config: config,
		pool:   pool,
	}, nil
}

func (b *ldapBackend) GetKeys(ctx context.Context, username string) [][]byte {
	result, err := b.pool.Search(ctx, b.config.Query.searchRequest(username))
	if err != nil {
		log.Printf("LDAP error: %v", err)
		return nil
	}

	var out [][]byte
	for _, ent := range result.Entries {
		k := []byte(ent.GetAttributeValue(b.config.Query.Attr))
		out = append(out, k)
	}
	return out
}
